import 'package:flutter/material.dart';

import 'package:sudoku/provider/data.dart';
import 'package:sudoku/utils/board_animate.dart';
import 'package:sudoku/utils/board_utils.dart';

class Cell {
  int value;
  bool isFixed;
  int conflictsCount = 0;

  bool isAnimated = false;

  Cell(
    this.value,
    this.isFixed,
  );

  /*
  * Build widget for board cell, with interactions
  */
  Container widget(Data myProvider, int row, int col) {
    final String imageAsset = getImageAssetName(myProvider);

    return Container(
      decoration: BoxDecoration(
        color: getBackgroundColor(myProvider),
        border: getCellBorders(myProvider, row, col),
      ),
      child: GestureDetector(
        child: AnimatedSwitcher(
          duration: const Duration(milliseconds: 100),
          transitionBuilder: (Widget child, Animation<double> animation) {
            return ScaleTransition(scale: animation, child: child);
          },
          child: Image(
            image: AssetImage(imageAsset),
            fit: BoxFit.fill,
            key: ValueKey<int>(imageAsset.hashCode),
          ),
        ),
        onTap: () {
          if (col != myProvider.currentCellCol || row != myProvider.currentCellRow) {
            myProvider.selectCell(col, row);
          } else {
            myProvider.selectCell(null, null);
          }
        },
      ),
    );
  }

  /*
  * Build widget for select/update value cell, with interactions
  */
  Container widgetUpdateValue(Data myProvider) {
    if (value < 0) {
      return Container();
    }

    final String imageAsset = getImageAssetName(myProvider);
    Color backgroundColor = Colors.grey.shade200;

    if (myProvider.showConflicts &&
        myProvider.currentCellCol != null &&
        myProvider.currentCellRow != null) {
      final List<List<Cell>> cells = myProvider.cells;
      final int blockSizeHorizontal = myProvider.blockSizeHorizontal;
      final int blockSizeVertical = myProvider.blockSizeVertical;

      if (!BoardUtils.isValueAllowed(cells, blockSizeHorizontal, blockSizeVertical,
          myProvider.currentCellCol, myProvider.currentCellRow, value)) {
        backgroundColor = Colors.pink.shade100;
      }
    }

    return Container(
        decoration: BoxDecoration(
          color: backgroundColor,
          border: Border.all(
            color: Colors.black,
            width: 2,
          ),
        ),
        child: GestureDetector(
          child: Image(image: AssetImage(imageAsset), fit: BoxFit.fill),
          onTap: () {
            if (myProvider.currentCellCol != null && myProvider.currentCellRow != null) {
              myProvider.updateCellValue(
                  myProvider.currentCellCol, myProvider.currentCellRow, value);
            }
            myProvider.selectCell(null, null);
            if (BoardUtils.checkBoardIsSolved(myProvider)) {
              BoardAnimate.startAnimation(myProvider, 'win');
            }
          },
        ));
  }

  /*
  * Compute image asset name, from skin and cell value/state
  */
  String getImageAssetName(Data myProvider) {
    if (value > 0) {
      int cellValue = myProvider.getTranslatedValueForDisplay(value);
      return 'assets/skins/${myProvider.parameterSkin}_$cellValue.png';
    }

    return 'assets/icons/cell_empty.png';
  }

  // Compute cell background color, from cell state
  Color getBackgroundColor(Data myProvider) {
    final Color editableCellColor = Colors.grey.shade100;
    final Color editableCellColorConflict = Colors.pink.shade100;
    final Color fixedCellColor = Colors.grey.shade300;
    final Color fixedCellColorConflict = Colors.pink.shade200;
    final Color editableSelectedValueColor = Colors.green.shade100;
    final Color fixedSelectedValueColor = Colors.green.shade300;
    final Color editableAnimated = Colors.green.shade200;
    final Color fixedAnimated = Colors.green.shade300;

    Color backgroundColor = editableCellColor;

    if (isFixed) {
      backgroundColor = fixedCellColor;
    }

    if (myProvider.showConflicts && (conflictsCount != 0)) {
      if (isFixed) {
        backgroundColor = fixedCellColorConflict;
      } else {
        backgroundColor = editableCellColorConflict;
      }
    }

    if (myProvider.showConflicts && (value == myProvider.currentCellValue)) {
      if (isFixed) {
        backgroundColor = fixedSelectedValueColor;
      } else {
        backgroundColor = editableSelectedValueColor;
      }
    }

    if (isAnimated) {
      if (isFixed) {
        backgroundColor = fixedAnimated;
      } else {
        backgroundColor = editableAnimated;
      }
    }

    return backgroundColor;
  }

  // Compute cell borders, from board size and cell state
  Border getCellBorders(Data myProvider, int row, int col) {
    final int blockSizeHorizontal = myProvider.blockSizeHorizontal;
    final int blockSizeVertical = myProvider.blockSizeVertical;

    const Color cellBorderDarkColor = Colors.black;
    const Color cellBorderLightColor = Colors.grey;
    const Color cellBorderSelectedColor = Colors.red;

    Color cellBorderColor = cellBorderSelectedColor;
    double cellBorderWidth = 4;

    // Reduce cell border width on big boards
    final int boardSize = blockSizeVertical * blockSizeHorizontal;
    if (boardSize > 8) {
      cellBorderWidth = 2;
      if (boardSize > 10) {
        cellBorderWidth = 1;
      }
    }

    if (!myProvider.gameIsRunning) {
      cellBorderColor = Colors.green.shade700;
    }

    Border borders = Border.all(
      color: cellBorderColor,
      width: cellBorderWidth,
    );

    // Update cell borders if not currently selected cell
    if (col != myProvider.currentCellCol || row != myProvider.currentCellRow) {
      borders = Border(
        top: BorderSide(
            width: cellBorderWidth,
            color:
                ((row % blockSizeVertical) == 0) ? cellBorderDarkColor : cellBorderLightColor),
        left: BorderSide(
            width: cellBorderWidth,
            color: ((col % blockSizeHorizontal) == 0)
                ? cellBorderDarkColor
                : cellBorderLightColor),
        right: BorderSide(
            width: cellBorderWidth,
            color: (((col + 1) % blockSizeHorizontal) == 0)
                ? cellBorderDarkColor
                : cellBorderLightColor),
        bottom: BorderSide(
            width: cellBorderWidth,
            color: (((row + 1) % blockSizeVertical) == 0)
                ? cellBorderDarkColor
                : cellBorderLightColor),
      );
    }

    return borders;
  }
}
